# -*- coding: utf-8 -*-

from .layout import Layout

DEFAULT = """__version__ = '{version}'
"""


class StandardLayout(Layout):

    def _create_default(self, path):
        package_path = path / self._package_name

        package_init = package_path / '__init__.py'

        package_path.mkdir()

        with package_init.open('w') as f:
            f.write(DEFAULT.format(version=self._version))



import contextlib
import hashlib
import os
import re
import tempfile
import shutil
import stat
import zipfile

from base64 import urlsafe_b64encode
from io import StringIO
from pathlib import Path
from types import SimpleNamespace

from poetry.__version__ import __version__
from poetry.semver.constraints import Constraint
from poetry.semver.constraints import MultiConstraint
from poetry.vcs import get_vcs
from poetry.version.helpers import format_python_constraint

from ..utils.helpers import normalize_file_permissions
from ..utils.tags import get_abbr_impl
from ..utils.tags import get_abi_tag
from ..utils.tags import get_impl_ver
from ..utils.tags import get_platform
from .builder import Builder


wheel_file_template = """\
Wheel-Version: 1.0
Generator: poetry {version}
Root-Is-Purelib: true
""".format(version=__version__)


class WheelBuilder(Builder):

    def __init__(self, poetry, io, target_fp, original=None):
        super().__init__(poetry, io)

        self._records = []
        self._original_path = self._path
        if original:
            self._original_path = original.file.parent

        # Open the zip file ready to write
        self._wheel_zip = zipfile.ZipFile(target_fp, 'w',
                                          compression=zipfile.ZIP_DEFLATED)

    @classmethod
    def make_in(cls, poetry, io, directory, original=None) -> SimpleNamespace:
        # We don't know the final filename until metadata is loaded, so write to
        # a temporary_file, and rename it afterwards.
        (fd, temp_path) = tempfile.mkstemp(suffix='.whl',
                                           dir=str(directory))
        try:
            with open(fd, 'w+b') as fp:
                wb = WheelBuilder(poetry, io, fp, original=original)
                wb.build()

            wheel_path = directory / wb.wheel_filename
            os.replace(temp_path, str(wheel_path))
        except:
            os.unlink(temp_path)
            raise

        return SimpleNamespace(builder=wb, file=wheel_path)

    @classmethod
    def make(cls, poetry, io) -> SimpleNamespace:
        """Build a wheel in the dist/ directory, and optionally upload it.
            """
        dist_dir = poetry.file.parent / 'dist'
        try:
            dist_dir.mkdir()
        except FileExistsError:
            pass

        return cls.make_in(poetry, io, dist_dir)

    def build(self) -> None:
        self._io.writeln(' - Building <info>wheel</info>')
        try:
            self._build()
            self.copy_module()
            self.write_metadata()
            self.write_record()
        finally:
            self._wheel_zip.close()

        self._io.writeln(f' - Built <fg=cyan>{self.wheel_filename}</>')

    def _build(self) -> None:
        if self._package.build:
            setup = self._path / 'setup.py'

            # We need to place ourselves in the temporary
            # directory in order to build the package
            current_path = os.getcwd()
            try:
                os.chdir(str(self._path))
                self._io.venv.run(
                    'python',
                    str(setup),
                    'build',
                    '-b', str(self._path / 'build')
                )
            finally:
                os.chdir(current_path)

            build_dir = self._path / 'build'
            lib = list(build_dir.glob('lib.*'))[0]
            for pkg in lib.glob('*'):
                shutil.rmtree(str(self._path / pkg.name))
                shutil.copytree(str(pkg), str(self._path / pkg.name))

    def copy_module(self) -> None:
        if self._module.is_package():
            files = self.find_files_to_add()

            # Walk the files and compress them,
            # sorting everything so the order is stable.
            for file in sorted(files):
                full_path = self._path / file

                # Do not include topmost files
                if full_path.relative_to(self._path) == Path(file.name):
                    continue

                self._add_file(full_path, file)
        else:
            self._add_file(str(self._module.path), self._module.path.name)

    def write_metadata(self):
        if 'scripts' in self._poetry.config or 'plugins' in self._poetry.config:
            with self._write_to_zip(self.dist_info + '/entry_points.txt') as f:
                self._write_entry_points(f)

        for base in ('COPYING', 'LICENSE'):
            for path in sorted(self._path.glob(base + '*')):
                self._add_file(path, '%s/%s' % (self.dist_info, path.name))

        with self._write_to_zip(self.dist_info + '/WHEEL') as f:
            self._write_wheel_file(f)

        with self._write_to_zip(self.dist_info + '/METADATA') as f:
            self._write_metadata_file(f)

    def write_record(self):
        # Write a record of the files in the wheel
        with self._write_to_zip(self.dist_info + '/RECORD') as f:
            for path, hash, size in self._records:
                f.write('{},sha256={},{}\n'.format(path, hash, size))
            # RECORD itself is recorded with no hash or size
            f.write(self.dist_info + '/RECORD,,\n')

    def find_excluded_files(self) -> list:
        # Checking VCS
        vcs = get_vcs(self._original_path)
        if not vcs:
            return []

        ignored = vcs.get_ignored_files()
        result = []
        for file in ignored:
            try:
                file = Path(file).absolute().relative_to(self._original_path)
            except ValueError:
                # Should only happen in tests
                continue

            result.append(file)

        return result

    @property
    def dist_info(self) -> str:
        return self.dist_info_name(self._package.name, self._package.version)

    @property
    def wheel_filename(self) -> str:
        if self._package.build:
            platform = get_platform().replace('.', '_').replace('-', '_')
            impl_name = get_abbr_impl()
            impl_ver = get_impl_ver()
            impl = impl_name + impl_ver
            abi_tag = str(get_abi_tag()).lower()
            tag = (impl, abi_tag, platform)
        else:
            platform = 'any'
            if self.supports_python2():
                impl = 'py2.py3'
            else:
                impl = 'py3'

            tag = (impl, 'none', platform)

        tag = '-'.join(tag)

        return '{}-{}-{}.whl'.format(
            re.sub("[^\w\d.]+", "_", self._package.pretty_name, flags=re.UNICODE),
            re.sub("[^\w\d.]+", "_", self._package.version, flags=re.UNICODE),
            tag
        )

    def supports_python2(self):
        return self._package.python_constraint.matches(
            MultiConstraint([
                Constraint('>=', '2.0.0'),
                Constraint('<', '3.0.0')
            ])
        )

    def dist_info_name(self, distribution, version) -> str:
        escaped_name = re.sub("[^\w\d.]+", "_", distribution, flags=re.UNICODE)
        escaped_version = re.sub("[^\w\d.]+", "_", version, flags=re.UNICODE)

        return '{}-{}.dist-info'.format(escaped_name, escaped_version)

    def _add_file(self, full_path, rel_path):
        full_path, rel_path = str(full_path), str(rel_path)
        if os.sep != '/':
            # We always want to have /-separated paths in the zip file and in
            # RECORD
            rel_path = rel_path.replace(os.sep, '/')

        zinfo = zipfile.ZipInfo.from_file(full_path, rel_path)

        # Normalize permission bits to either 755 (executable) or 644
        st_mode = os.stat(full_path).st_mode
        new_mode = normalize_file_permissions(st_mode)
        zinfo.external_attr = (new_mode & 0xFFFF) << 16  # Unix attributes

        if stat.S_ISDIR(st_mode):
            zinfo.external_attr |= 0x10  # MS-DOS directory flag

        hashsum = hashlib.sha256()
        with open(full_path, 'rb') as src, self._wheel_zip.open(zinfo,
                                                                'w') as dst:
            while True:
                buf = src.read(1024 * 8)
                if not buf:
                    break
                hashsum.update(buf)
                dst.write(buf)

        size = os.stat(full_path).st_size
        hash_digest = urlsafe_b64encode(hashsum.digest()).decode(
            'ascii').rstrip('=')

        self._records.append((rel_path, hash_digest, size))

    @contextlib.contextmanager
    def _write_to_zip(self, rel_path):
        sio = StringIO()
        yield sio

        # The default is a fixed timestamp rather than the current time, so
        # that building a wheel twice on the same computer can automatically
        # give you the exact same result.
        date_time = (2016, 1, 1, 0, 0, 0)
        zi = zipfile.ZipInfo(rel_path, date_time)
        b = sio.getvalue().encode('utf-8')
        hashsum = hashlib.sha256(b)
        hash_digest = urlsafe_b64encode(
            hashsum.digest()
        ).decode('ascii').rstrip('=')

        self._wheel_zip.writestr(zi, b, compress_type=zipfile.ZIP_DEFLATED)
        self._records.append((rel_path, hash_digest, len(b)))

    def _write_entry_points(self, fp):
        """
        Write entry_points.txt.
        """
        entry_points = self.convert_entry_points()

        for group_name in sorted(entry_points):
            fp.write('[{}]\n'.format(group_name))
            for ep in sorted(entry_points[group_name]):
                fp.write(ep.replace(' ', ''))

            fp.write('\n')

    def _write_wheel_file(self, fp):
        fp.write(wheel_file_template)

        if self.supports_python2():
            fp.write("Tag: py2-none-any\n")

        fp.write("Tag: py3-none-any\n")

    def _write_metadata_file(self, fp):
        """
        Write out metadata in the 1.x format (email like)
        """
        fp.write('Metadata-Version: 1.2\n')
        fp.write(f'Name: {self._package.name}\n')
        fp.write(f'Version: {self._package.version}\n')
        fp.write(f'Summary: {self._package.description}\n')
        fp.write(f'Home-page: {self._package.homepage or self._package.repository_url or "UNKNOWN"}\n')
        fp.write(f'License: {self._package.license or "UNKOWN"}\n')

        # Optional fields
        if self._package.keywords:
            fp.write(f"Keywords: {','.join(self._package.keywords)}\n")

        if self._package.authors:
            author = self.convert_author(self._package.authors[0])

            fp.write(f'Author: {author["name"]}\n')
            fp.write(f'Author-email: {author["email"]}\n')

        if self._package.python_versions != '*':
            python_requires = format_python_constraint(self._package.python_constraint)

            fp.write(f'Requires-Python: {python_requires}\n')

        classifiers = self.get_classifers()
        for classifier in classifiers:
            fp.write(f'Classifier: {classifier}\n')

        for dep in self._package.requires:
            fp.write('Requires-Dist: {}\n'.format(dep.to_pep_508()))

        if self._package.readme is not None:
            fp.write('\n' + self._package.readme + '\n')



from .about import AboutCommand
from .add import AddCommand
from .build import BuildCommand
from .config import ConfigCommand
from .install import InstallCommand
from .lock import LockCommand
from .new import NewCommand
from .publish import PublishCommand
from .remove import RemoveCommand
from .show import ShowCommand
from .update import UpdateCommand



import hashlib

import toml
import requests

from pathlib import Path

from poetry.locations import CONFIG_DIR
from poetry.semver.constraints import Constraint
from poetry.semver.constraints import MultiConstraint

from .builders.builder import Builder


class Publisher:

    def __init__(self, poetry, io):
        self._poetry = poetry
        self._package = poetry.package
        self._io = io

    def publish(self, repository_name):
        if repository_name:
            self._io.writeln(
                f'Publishing <info>{self._package.pretty_name}</info> '
                f'(<comment>{self._package.pretty_version}</comment>) '
                f'to <fg=cyan>{repository_name}</>'
            )
        else:
            self._io.writeln(
                f'Publishing <info>{self._package.pretty_name}</info> '
                f'(<comment>{self._package.pretty_version}</comment>) '
                f'to <fg=cyan>PyPI</>'
            )

        if not repository_name:
            url = 'https://upload.pypi.org/legacy/'
        else:
            # Retrieving config information
            config_file = Path(CONFIG_DIR) / 'config.toml'

            if not config_file.exists():
                raise RuntimeError(
                    'Config file does not exist. '
                    'Unable to get repository information'
                )

            with config_file.open() as f:
                config = toml.loads(f.read())

            if (
                'repositories' not in config
                or repository_name not in config['repositories']
            ):
                raise RuntimeError(
                    f'Repository {repository_name} is not defined'
                )

            url = config['repositories'][repository_name]['url']

        username = None
        password = None
        auth_file = Path(CONFIG_DIR) / 'auth.toml'
        if not auth_file.exists():
            # No auth file, we will ask for info later
            auth_config = {}
        else:
            with auth_file.open() as f:
                auth_config = toml.loads(f.read())

            if 'http-basic' in auth_config and repository_name in auth_config['http-basic']:
                config = auth_config['http-basic'][repository_name]

                username = config.get('username')
                password = config.get('password')

        return self.upload(url, username=username, password=password)

    def upload(self, url, username=None, password=None):
        data = self.build_post_data('file_upload')

    def upload_file(self, file, url, username, password):
        data = self.build_post_data('file_upload')

        data['protocol_version'] = '1'
        if file.suffix == '.whl':
            data['filetype'] = 'bdist_wheel'
            py2_support = self._package.python_constraint.matches(
                MultiConstraint([
                    Constraint('>=', '2.0.0'),
                    Constraint('<', '3.0.0')
                ])
            )
            data['pyversion'] = ('py2.' if py2_support else '') + 'py3'
        else:
            data['filetype'] = 'sdist'

        with file.open('rb') as f:
            content = f.read()
            files = {'content': (file.name, content)}
            data['md5_digest'] = hashlib.md5(content).hexdigest()

        log.info('Uploading %s...', file)
        resp = requests.post(repo['url'],
                             data=data,
                             files=files,
                             auth=(repo['username'], repo['password']),
                             )
        resp.raise_for_status()

    def build_post_data(self, action):
        builder = Builder(self._poetry, self._io)

        d = {
            ":action": action,

            "name": self._package.name,
            "version": self._package.version,

            # additional meta-data
            "metadata_version": '1.2',
            "summary": self._package.description,
            "home_page": self._package.homepage or self._package.repository_url,
            "author": self._package.author_name,
            "author_email": self._package.author_email,
            "maintainer": self._package.author_name,
            "maintainer_email": self._package.author_email,
            "license": self._package.license,
            "description": self._package.readme,
            "keywords": ','.join(self._package.keywords),
            "platform": None if self._package.platform == '*' else self._package.platform,
            "classifiers": builder.get_classifers(),
            "download_url": None,
            "supported_platform": None if self._package.platform == '*' else self._package.platform,
            "project_urls": [],
            "provides_dist": [],
            "obsoletes_dist": [],
            "requires_dist": [d.to_pep_508() for d in self._package.requires],
            "requires_external": [],
            "requires_python": builder.convert_python_version(),
        }

        return {k: v for k, v in d.items() if v}



from cleo import Command as BaseCommand
from cleo.inputs import ListInput

from poetry.poetry import Poetry

from ..styles.poetry import PoetryStyle


class Command(BaseCommand):

    @property
    def poetry(self) -> Poetry:
        return self.get_application().poetry

    def reset_poetry(self) -> None:
        self.get_application().reset_poetry()

    def call(self, name, options=None):
        """
        Call another command.

        Fixing style being passed rather than an output
        """
        if options is None:
            options = []

        command = self.get_application().find(name)

        options = [('command', command.get_name())] + options

        return command.run(ListInput(options), self.output.output)

    def run(self, i, o) -> int:
        """
        Initialize command.
        """
        self.input = i
        self.output = PoetryStyle(i, o, self.get_application().venv)

        return super(BaseCommand, self).run(i, o)



import os

from cleo import Application as BaseApplication

from poetry.poetry import Poetry
from poetry.utils.venv import Venv

from .commands import AboutCommand
from .commands import AddCommand
from .commands import BuildCommand
from .commands import ConfigCommand
from .commands import InstallCommand
from .commands import LockCommand
from .commands import NewCommand
from .commands import PublishCommand
from .commands import RemoveCommand
from .commands import ShowCommand
from .commands import UpdateCommand


class Application(BaseApplication):

    def __init__(self):
        super().__init__('Poetry', Poetry.VERSION)

        self._poetry = None
        self._venv = Venv.create()

    @property
    def poetry(self) -> Poetry:
        if self._poetry is not None:
            return self._poetry

        self._poetry = Poetry.create(os.getcwd())

        return self._poetry

    def reset_poetry(self) -> None:
        self._poetry = None

    @property
    def venv(self) -> Venv:
        return self._venv

    def get_default_commands(self) -> list:
        commands = super(Application, self).get_default_commands()

        return commands + [
            AboutCommand(),
            AddCommand(),
            BuildCommand(),
            ConfigCommand(),
            InstallCommand(),
            LockCommand(),
            NewCommand(),
            PublishCommand(),
            RemoveCommand(),
            ShowCommand(),
            UpdateCommand(),
        ]

    def do_run(self, i, o) -> int:
        if self._venv.is_venv() and o.is_verbose():
            o.writeln(f'Using virtualenv: <comment>{self._venv.venv}</>')

        return super().do_run(i, o)



from poetry.console.styles.poetry import PoetryStyle
from poetry.utils.venv import Venv


class NullVenv(Venv):

    def __init__(self, execute=False):
        super().__init__()

        self.executed = []
        self._execute = execute

    def run(self, bin: str, *args):
        self.executed.append([bin] + list(args))

        if self._execute:
            return super().run(bin, *args)

    def _bin(self, bin):
        return bin


class NullIO(PoetryStyle):

    def __init__(self, execute=False):
        self._venv = NullVenv(execute=execute)

    @property
    def venv(self) -> NullVenv:
        return self._venv

    def is_quiet(self) -> bool:
        return False

    def is_verbose(self) -> bool:
        return False

    def is_very_verbose(self) -> bool:
        return False

    def is_debug(self) -> bool:
        return False

    def writeln(self, *args, **kwargs):
        pass

    def write(self, *args, **kwargs):
        pass

    def new_line(self, *args, **kwargs):
        pass



from poetry.semver.constraints import MultiConstraint
from poetry.utils.helpers import canonicalize_name


class Metadata:

    metadata_version = '1.2'
    # version 1.0
    name = None
    version = None
    platforms = ()
    supported_platforms = ()
    summary = None
    description = None
    keywords = None
    home_page = None
    download_url = None
    author = None
    author_email = None
    license = None
    # version 1.1
    classifiers = ()
    requires = ()
    provides = ()
    obsoletes = ()
    # version 1.2
    maintainer = None
    maintainer_email = None
    requires_python = None
    requires_external = ()
    requires_dist = ()
    provides_dist = ()
    obsoletes_dist = ()
    project_urls = ()

    @classmethod
    def from_package(cls, package) -> 'Metadata':
        meta = cls()

        meta.name = canonicalize_name(package.name)
        meta.version = package.version
        meta.summary = package.description
        meta.description = package.readme
        meta.keywords = ','.join(package.keywords)
        meta.home_page = package.homepage or package.repository_url
        meta.author = package.author_name
        meta.author_email = package.author_email
        meta.license = package.license
        meta.classifiers = package.classifiers

        # Version 1.2
        meta.maintainer = meta.author
        meta.maintainer_email = meta.author_email
        meta.requires_python = package.python_constraint
        meta.requires_dist = [d.to_pep_508() for d in package.requires]

        # Requires python
        constraint = package.python_constraint
        if isinstance(constraint, MultiConstraint):
            python_requires = ','.join(
                [str(c).replace(' ', '') for c in constraint.constraints]
            )
        else:
            python_requires = str(constraint).replace(' ', '')

        meta.requires_python = python_requires

        return meta



import glob
import os
import subprocess
import sys

from subprocess import CalledProcessError


class VenvError(Exception):

    pass


class VenvCommandError(VenvError):

    def __init__(self, e: CalledProcessError):
        message = f'Command {e.cmd} errored with the following output: \n' \
                  f'{e.output.decode()}'

        super().__init__(message)


class Venv:

    def __init__(self, venv=None):
        self._venv = venv
        self._version_info = None

    @classmethod
    def create(cls) -> 'Venv':
        if 'VIRTUAL_ENV' not in os.environ:
            # Not in a virtualenv
            return cls()

        # venv detection:
        # stdlib venv may symlink sys.executable, so we can't use realpath.
        # but others can symlink *to* the venv Python,
        # so we can't just use sys.executable.
        # So we just check every item in the symlink tree (generally <= 3)
        p = os.path.normcase(sys.executable)
        paths = [p]
        while os.path.islink(p):
            p = os.path.normcase(
                os.path.join(os.path.dirname(p), os.readlink(p)))
            paths.append(p)

        p_venv = os.path.normcase(os.environ['VIRTUAL_ENV'])
        if any(p.startswith(p_venv) for p in paths):
            # Running properly in the virtualenv, don't need to do anything
            return cls()

        if sys.platform == "win32":
            venv = os.path.join(
                os.environ['VIRTUAL_ENV'], 'Lib', 'site-packages'
            )
        else:
            lib = os.path.join(
                os.environ['VIRTUAL_ENV'], 'lib'
            )

            python = glob.glob(
                os.path.join(lib, 'python*')
            )[0].replace(
                lib + '/', ''
            )

            venv = os.path.join(
                lib,
                python,
                'site-packages'
            )

        return cls(venv)

    @property
    def venv(self):
        return self._venv

    @property
    def python(self) -> str:
        """
        Path to current python executable
        """
        return self._bin('python')

    @property
    def pip(self) -> str:
        """
        Path to current pip executable
        """
        return self._bin('pip')

    @property
    def version_info(self):
        if self._version_info is not None:
            return self._version_info

        if not self.is_venv():
            self._version_info = sys.version_info
        else:
            output = self.run('python', '--version')

            version = output.split(' ')
            self._version_info = version[1].strip().split('.')

        return self._version_info

    def run(self, bin: str, *args, **kwargs) -> str:
        """
        Run a command inside the virtual env.
        """
        cmd = [self._bin(bin)] + list(args)

        try:
            output = subprocess.check_output(
                cmd, stderr=subprocess.STDOUT,
                **kwargs
            )
        except CalledProcessError as e:
            raise VenvCommandError(e)

        return output.decode()

    def _bin(self, bin) -> str:
        """
        Return path to the given executable.
        """
        if not self.is_venv():
            return bin

        return os.path.realpath(
            os.path.join(self._venv, '..', '..', '..', 'bin', bin)
        )

    def is_venv(self) -> bool:
        return self._venv is not None



import json
import re

from pathlib import Path

from poetry.locations import CONFIG_DIR
from poetry.toml import loads

from .command import Command


TEMPLATE = """[repositories]
"""

AUTH_TEMPLATE = """[http-basic]
"""


class ConfigCommand(Command):
    """
    Sets/Gets config options.

    config
        { key : Setting key. }
        { value?* : Setting value. }
        { --list : List configuration settings }
        { --unset : Unset configuration setting }
    """

    help = """This command allows you to edit the poetry config settings and repositories..

To add a repository:

    <comment>poetry repositories.foo https://bar.com/simple/</comment>

To remove a repository (repo is a short alias for repositories):

    <comment>poetry --unset repo.foo</comment>
"""

    def __init__(self):
        super().__init__()

        self._config_file = None
        self._config = {}

        self._auth_config_file = None
        self._auth_config = {}

    def initialize(self, i, o):
        super().initialize(i, o)

        # Create config file if it does not exist
        self._config_file = Path(CONFIG_DIR) / 'config.toml'
        self._auth_config_file = Path(CONFIG_DIR) / 'auth.toml'

        if not self._config_file.exists():
            self._config_file.parent.mkdir(parents=True, exist_ok=True)
            self._config_file.write_text(TEMPLATE)
        if not self._auth_config_file.exists():

            self._auth_config_file.parent.mkdir(parents=True, exist_ok=True)
            self._auth_config_file.write_text(AUTH_TEMPLATE)

        with self._config_file.open() as f:
            self._config = loads(f.read())

        with self._auth_config_file.open() as f:
            self._auth_config = loads(f.read())

    def handle(self):
        if self.option('list'):
            self._list_configuration(self._config)

            return 0

        setting_key = self.argument('key')
        if not setting_key:
            return 0

        if self.argument('value') and self.option('unset'):
            raise RuntimeError('You can not combine a setting value with --unset')

        # show the value if no value is provided
        if not self.argument('value') and not self.option('unset'):
            m = re.match('^repos?(?:itories)?(?:\.(.+))?', self.argument('key'))
            if m:
                if not m.group(1):
                    value = {}
                    if 'repositories' in self._config:
                        value = self._config['repositories']
                else:
                    if m.group(1) not in self._config['repositories']:
                        raise ValueError(
                            f'There is not {m.group(1)} repository defined'
                        )

                    value = self._config['repositories'][m.group(1)]

                self.line(str(value))

            return 0

        values = self.argument('value')

        # handle repositories
        m = re.match('^repos?(?:itories)?(?:\.(.+))?', self.argument('key'))
        if m:
            if not m.group(1):
                raise ValueError('You cannot remove the [repositories] section')

            if self.option('unset'):
                if m.group(1) not in self._config['repositories']:
                    raise ValueError(f'There is not {m.group(1)} repository defined')

                del self._config[m.group(1)]

                self._config_file.write_text(self._config.dumps())

                return 0

            if len(values) == 1:
                url = values[0]

                if m.group(1) in self._config['repositories']:
                    self._config['repositories'][m.group(1)]['url'] = url
                else:
                    self._config['repositories'][m.group(1)] = {
                        'url': url
                    }

                self._config_file.write_text(self._config.dumps())

                return 0

            raise ValueError(
                'You must pass the url. '
                'Example: poetry config repositories.foo https://bar.com'
            )

        # handle auth
        m = re.match('^(http-basic)\.(.+)', self.argument('key'))
        if m:
            if self.option('unset'):
                if m.group(2) not in self._auth_config[m.group(1)]:
                    raise ValueError(
                        f'There is no {m.group(2)} {m.group(1)} defined'
                    )

                del self._auth_config[m.group(1)][m.group(2)]

                self._auth_config_file.write_text(self._auth_config.dumps())

                return 0

            if m.group(1) == 'http-basic':
                if len(values) == 1:
                    username = values[0]
                    # Only username, so we prompt for password
                    password = self.secret('Password:')
                elif len(values) != 2:
                    raise ValueError(f'Expected one or two arguments '
                                     f'(username, password), got {len(values)}')
                else:
                    username = values[0]
                    password = values[1]

                self._auth_config[m.group(1)][m.group(2)] = {
                    'username': username,
                    'password': password
                }

            self._auth_config_file.write_text(self._auth_config.dumps())

            return 0

        raise ValueError(f'Setting {self.argument("key")} does not exist')

    def _list_configuration(self, contents, k=None):
        orig_k = k

        for key, value in contents.items():
            if k is None and key not in ['config', 'repositories']:
                continue

            if isinstance(value, dict) or key == 'repositories' and k is None:
                if k is None:
                    k = ''

                k += re.sub('^config\.', '', key + '.')
                self._list_configuration(value, k=k)
                k = orig_k

                continue

            if isinstance(value, list):
                value = [
                    json.dumps(val) if isinstance(val, list) else val
                    for val in value
                ]

                value = f'[{", ".join(value)}]'

            value = json.dumps(value)

            self.line(f'[<comment>{(k or "") + key}</comment>] <info>{value}</info>')



from distutils.core import setup

packages = \
['poetry',
 'poetry.console',
 'poetry.console.commands',
 'poetry.console.styles',
 'poetry.installation',
 'poetry.io',
 'poetry.layouts',
 'poetry.masonry',
 'poetry.masonry.builders',
 'poetry.masonry.publishing',
 'poetry.masonry.utils',
 'poetry.mixology',
 'poetry.mixology.contracts',
 'poetry.mixology.graph',
 'poetry.packages',
 'poetry.packages.constraints',
 'poetry.puzzle',
 'poetry.puzzle.operations',
 'poetry.repositories',
 'poetry.semver',
 'poetry.semver.constraints',
 'poetry.toml',
 'poetry.toml.prettify',
 'poetry.toml.prettify.elements',
 'poetry.toml.prettify.elements.traversal',
 'poetry.toml.prettify.lexer',
 'poetry.toml.prettify.parser',
 'poetry.toml.prettify.tokens',
 'poetry.utils',
 'poetry.vcs',
 'poetry.version']

package_data = \
{'': ['*']}

install_requires = \
['cleo (>=0.6.0.0,<0.7.0.0)',
 'requests (>=2.18.0.0,<3.0.0.0)',
 'toml (>=0.9.0.0,<0.10.0.0)',
 'cachy (>=0.1.0.0,<0.2.0.0)',
 'pip-tools (>=1.11.0.0,<2.0.0.0)',
 'requests-toolbelt (>=0.8.0.0,<0.9.0.0)']

entry_points = \
{'console_scripts': ['poetry = poetry:console.run']}

setup_kwargs = {
    'name': 'poetry',
    'version': '0.5.0b2',
    'description': 'Python dependency management and packaging made easy.',
    'long_description': 'Poetry: Dependency Management for Python\n========================================\n\n.. figure:: https://travis-ci.org/sdispater/poetry.svg\n   :alt: Poetry build status\n\n   Poetry build status\n\nPoetry helps you declare, manage and install dependencies of Python\nprojects, ensuring you have the right stack everywhere.\n\n.. figure:: https://raw.githubusercontent.com/sdispater/poetry/master/assets/install.gif\n   :alt: Poet Install\n\n   Poet Install\n\nThe package is **highly experimental** at the moment so expect things to\nchange and break. However, if you feel adventurous feedback and pull\nrequests are greatly appreciated.\n\nAlso, be aware that the features described here are the goal that this\nlibrary is aiming for and, as of now, not all of them are implemented\n(but most of them are). The dependency management is done while the\npackaging and publishing are still at the beginning.\n\nAnd finally, Poetry\'s code is only compatible with Python 3.6+ but it\ncan manage Python project\'s with previous versions without any problem.\n\nInstallation\n------------\n\n.. code:: bash\n\n    pip install poetry\n\nEnable tab completion for Bash, Fish, or Zsh\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n``poetry`` supports generating completion scripts for Bash, Fish, and\nZsh. See ``poet help completions`` for full details, but the gist is as\nsimple as using one of the following:\n\n.. code:: bash\n\n    # Bash\n    poetry completions bash > /etc/bash_completion.d/pyproject.bash-completion\n\n    # Bash (macOS/Homebrew)\n    poetry completions bash > $(brew --prefix)/etc/bash_completion.d/pyproject.bash-completion\n\n    # Fish\n    poetry completions fish > ~/.config/fish/completions/pyproject.fish\n\n    # Zsh\n    poetry completions zsh > ~/.zfunc/_poetry\n\n*Note:* you may need to restart your shell in order for the changes to\ntake effect.\n\nFor ``zsh``, you must then add the following line in your ``~/.zshrc``\nbefore ``compinit``:\n\n.. code:: zsh\n\n    fpath+=~/.zfunc\n\nIntroduction\n------------\n\n``poetry`` is a tool to handle dependencies installation, building and\npackaging of Python packages. It only needs one file to do all of that:\nthe new, `standardized <https://www.python.org/dev/peps/pep-0518/>`__\n``pyproject.toml``.\n\n.. code:: toml\n\n    [tool.poetry]\n    name = "my-package"\n    version = "0.1.0"\n    description = "The description of the package"\n\n    license = "MIT"\n\n    authors = [\n        "Sébastien Eustace <sebastien@eustace.io>"\n    ]\n\n    readme = \'README.md\'\n\n    repository = "https://github.com/sdispater/poetry"\n    homepage = "https://github.com/sdispater/poetry"\n\n    keywords = [\'packaging\', \'poetry\']\n\n    [tool.poetry.dependencies]\n    python = "~2.7 || ^3.2"  # Compatible python versions must be declared here\n    toml = "^0.9"\n    # Dependencies with extras\n    requests = { version = "^2.13", extras = [ "security" ] }\n    # Python specific dependencies with prereleases allowed\n    pathlib2 = { version = "^2.2", python = "~2.7", allows_prereleases = true }\n    # Git dependencies\n    cleo = { git = "https://github.com/sdispater/cleo.git", branch = "master" }\n\n    # Optional dependencies (extras)\n    pendulum = { version = "^1.4", optional = true }\n\n    [tool.poetry.dev-dependencies]\n    pytest = "^3.0"\n    pytest-cov = "^2.4"\n\n    [tool.poetry.scripts]\n    my-script = \'my_package:main\'\n\nThere are some things we can notice here:\n\n-  It will try to enforce `semantic versioning <http://semver.org>`__ as\n   the best practice in version naming.\n-  You can specify the readme, included and excluded files: no more\n   ``MANIFEST.in``. ``poetry`` will also use VCS ignore files (like\n   ``.gitignore``) to populate the ``exclude`` section.\n-  Keywords (up to 5) can be specified and will act as tags on the\n   packaging site.\n-  The dependencies sections support caret, tilde, wildcard, inequality\n   and multiple requirements.\n-  You must specify the python versions for which your package is\n   compatible.\n\n``poetry`` will also detect if you are inside a virtualenv and install\nthe packages accordingly. So, ``poetry`` can be installed globally and\nused everywhere.\n\n``poetry`` also comes with a full fledged dependency resolution library,\ninspired by `Molinillo <https://github.com/CocoaPods/Molinillo>`__.\n\nWhy?\n----\n\nPackaging system and dependency management in Python is rather\nconvoluted and hard to understand for newcomers. Even for seasoned\ndevelopers it might be cumbersome at times to create all files needed in\na Python project: ``setup.py``, ``requirements.txt``, ``setup.cfg``,\n``MANIFEST.in`` and the newly added ``Pipfile``.\n\nSo I wanted a tool that would limit everything to a single configuration\nfile to do: dependency management, packaging and publishing.\n\nIt takes inspiration in tools that exist in other languages, like\n``composer`` (PHP) or ``cargo`` (Rust).\n\nAnd, finally, there is no reliable tool to properly resolves\ndependencies in Python, so I started ``poetry`` to bring an exhaustive\ndepency resolver to the Python community.\n\nWhat about Pipenv?\n~~~~~~~~~~~~~~~~~~\n\nIn short: I do not like the CLI it provides, or some of the decisions\nmade, and I think we can do a better and more intuitive one. Here are a\nfew things that I don\'t like.\n\nDependency resolution\n^^^^^^^^^^^^^^^^^^^^^\n\nThe dependency resolution is erratic and will fail even is there is a\nsolution. Let\'s take an example:\n\n.. code:: bash\n\n    pipenv install oslo.utils==1.4.0\n\nwill fail with this error:\n\n.. code:: text\n\n    Could not find a version that matches pbr!=0.7,!=2.1.0,<1.0,>=0.6,>=2.0.0\n\nwhile Poetry will get you the right set of packages:\n\n.. code:: bash\n\n    poetry add oslo.utils=1.4.0\n\nresults in :\n\n.. code:: text\n\n      - Installing pytz (2018.3)\n      - Installing netifaces (0.10.6)\n      - Installing netaddr (0.7.19)\n      - Installing oslo.i18n (2.1.0)\n      - Installing iso8601 (0.1.12)\n      - Installing six (1.11.0)\n      - Installing babel (2.5.3)\n      - Installing pbr (0.11.1)\n      - Installing oslo.utils (1.4.0)\n\nInstall command\n^^^^^^^^^^^^^^^\n\nWhen you specify a package to the ``install`` command it will add it as\na wildcard dependency. This means that **any** version of this package\ncan be installed which can lead to compatibility issues.\n\nAlso, you have to explicitely tell it to not update the locked packages\nwhen you installed new ones. This should be the default.\n\nRemove command\n^^^^^^^^^^^^^^\n\nThe ``remove`` command will only remove the package specified but not\nits dependencies if they are no longer needed.\n\nYou either have to use ``sync`` or ``clean`` to fix that.\n\nToo limited in scope\n^^^^^^^^^^^^^^^^^^^^\n\nFinally, the ``Pipfile`` is just a replacement from ``requirements.txt``\nand, in the end, you will still need to populate your ``setup.py`` file\n(or ``setup.cfg``) with the exact same dependencies you declared in your\n``Pipfile``. So, in the end, you will still need to manage a few\nconfiguration files to properly setup your project.\n\nCommands\n--------\n\nnew\n~~~\n\nThis command will help you kickstart your new Python project by creating\na directory structure suitable for most projects.\n\n.. code:: bash\n\n    poetry new my-package\n\nwill create a folder as follows:\n\n.. code:: text\n\n    my-package\n    ├── pyproject.toml\n    ├── README.rst\n    ├── my_package\n        └── __init__.py\n    └── tests\n        ├── __init__.py\n        └── test_my_package\n\nIf you want to name your project differently than the folder, you can\npass the ``--name`` option:\n\n.. code:: bash\n\n    poetry new my-folder --name my-package\n\ninstall\n~~~~~~~\n\nThe ``install`` command reads the ``pyproject.toml`` file from the\ncurrent directory, resolves the dependencies, and installs them.\n\n.. code:: bash\n\n    poetry install\n\nIf there is a ``pyproject.lock`` file in the current directory, it will\nuse the exact versions from there instead of resolving them. This\nensures that everyone using the library will get the same versions of\nthe dependencies.\n\nIf there is no ``pyproject.lock`` file, Poetry will create one after\ndependency resolution.\n\nYou can specify to the command that you do not want the development\ndependencies installed by passing the ``--no-dev`` option.\n\n.. code:: bash\n\n    poetry install --no-dev\n\nYou can also specify the extras you want installed by passing the\n``--E|--extras`` option (See `Extras <#extras>`__ for more info)\n\n.. code:: bash\n\n    poetry install --extras "mysql pgsql"\n    poetry install -E mysql -E pgsql\n\nOptions\n^^^^^^^\n\n-  ``--no-dev``: Do not install dev dependencies.\n-  ``-E|--extras``: Features to install (multiple values allowed).\n\nupdate\n~~~~~~\n\nIn order to get the latest versions of the dependencies and to update\nthe ``pyproject.lock`` file, you should use the ``update`` command.\n\n.. code:: bash\n\n    poetry update\n\nThis will resolve all dependencies of the project and write the exact\nversions into ``pyproject.lock``.\n\nIf you just want to update a few packages and not all, you can list them\nas such:\n\n.. code:: bash\n\n    poetry update requests toml\n\nOptions\n^^^^^^^\n\n-  ``--dry-run`` : Outputs the operations but will not execute anything\n   (implicitly enables --verbose).\n\nadd\n~~~\n\nThe ``add`` command adds required packages to your ``pyproject.toml``\nand installs them.\n\nIf you do not specify a version constraint, poetry will choose a\nsuitable one based on the available package versions.\n\n.. code:: bash\n\n    poetry add requests pendulum\n\nOptions\n^^^^^^^\n\n-  ``--D|dev``: Add package as development dependency.\n-  ``--optional`` : Add as an optional dependency.\n-  ``--dry-run`` : Outputs the operations but will not execute anything\n   (implicitly enables --verbose).\n\nremove\n~~~~~~\n\nThe ``remove`` command removes a package from the current list of\ninstalled packages\n\n.. code:: bash\n\n    poetry remove pendulum\n\nOptions\n^^^^^^^\n\n-  ``--D|dev``: Removes a package from the development dependencies.\n-  ``--dry-run`` : Outputs the operations but will not execute anything\n   (implicitly enables --verbose).\n\nshow\n~~~~\n\nTo list all of the available packages, you can use the ``show`` command.\n\n.. code:: bash\n\n    poetry show\n\nIf you want to see the details of a certain package, you can pass the\npackage name.\n\n.. code:: bash\n\n    poetry show pendulum\n\n    name        : pendulum\n    version     : 1.4.2\n    description : Python datetimes made easy\n\n    dependencies:\n     - python-dateutil >=2.6.1\n     - tzlocal >=1.4\n     - pytzdata >=2017.2.2\n\nOptions\n^^^^^^^\n\n-  ``--tree``: List the dependencies as a tree.\n-  ``-l|--latest``: Show the latest version.\n-  ``-o|--outdated``: Show the latest version but only for packages that\n   are outdated.\n\nbuild\n~~~~~\n\nThe ``build`` command builds the source and wheels archives.\n\n.. code:: bash\n\n    poetry build\n\nNote that, at the moment, only pure python wheels are supported.\n\nOptions\n^^^^^^^\n\n-  ``-F|--format``: Limit the format to either wheel or sdist.\n\npublish\n~~~~~~~\n\nThis command builds (if not already built) and publishes the package to\nthe remote repository.\n\nIt will automatically register the package before uploading if this is\nthe first time it is submitted.\n\n.. code:: bash\n\n    poetry publish\n\nOptions\n^^^^^^^\n\n-  ``-r|--repository``: The repository to register the package to\n   (default: ``pypi``). Should match a repository name set by the\n   ```config`` <#config>`__ command.\n\n``config``\n~~~~~~~~~~\n\nThe ``config`` command allows you to edit poetry config settings and\nrepositories.\n\n.. code:: bash\n\n    poetry config --list\n\nUsage\n^^^^^\n\n.. code:: bash\n\n    poetry config [options] [setting-key] [setting-value1] ... [setting-valueN]\n\n``setting-key`` is a configuration option name and ``setting-value1`` is\na configuration value.\n\nModifying repositories\n^^^^^^^^^^^^^^^^^^^^^^\n\nIn addition to modifying the config section, the config command also\nsupports making changes to the repositories section by using it the\nfollowing way:\n\n.. code:: bash\n\n    poetry config repositories.foo https://foo.bar/simple/\n\nThis will set the url for repository ``foo`` to\n``https://foo.bar/simple/``.\n\nIf you want to store your credentials for a specific repository, you can\ndo so easily:\n\n.. code:: bash\n\n    poetry config http-basic.foo username password\n\nIf you do not specify the password you will be prompted to write it.\n\nOptions\n^^^^^^^\n\n-  ``--unset``: Remove the configuration element named by\n   ``setting-key``.\n-  ``--list``: Show the list of current config variables.\n\nsearch\n~~~~~~\n\nThis command searches for packages on a remote index.\n\n.. code:: bash\n\n    poetry search requests pendulum\n\nOptions\n^^^^^^^\n\n-  ``-N|--only-name``: Search only in name.\n\nlock\n~~~~\n\nThis command locks (without installing) the dependencies specified in\n``pyproject.toml``.\n\n.. code:: bash\n\n    poetry lock\n\nThe ``pyproject.toml`` file\n---------------------------\n\nThe ``tool.poetry`` section of the ``pyproject.toml`` file is composed\nof multiple sections.\n\nname\n~~~~\n\nThe name of the package. **Required**\n\nversion\n~~~~~~~\n\nThe version of the package. **Required**\n\nThis should follow `semantic versioning <http://semver.org/>`__. However\nit will not be enforced and you remain free to follow another\nspecification.\n\ndescription\n~~~~~~~~~~~\n\nA short description of the package. **Required**\n\nlicense\n~~~~~~~\n\nThe license of the package.\n\nThe recommended notation for the most common licenses is (alphabetical):\n\n-  Apache-2.0\n-  BSD-2-Clause\n-  BSD-3-Clause\n-  BSD-4-Clause\n-  GPL-2.0\n-  GPL-2.0+\n-  GPL-3.0\n-  GPL-3.0+\n-  LGPL-2.1\n-  LGPL-2.1+\n-  LGPL-3.0\n-  LGPL-3.0+\n-  MIT\n\nOptional, but it is highly recommended to supply this. More identifiers\nare listed at the `SPDX Open Source License\nRegistry <https://www.spdx.org/licenses/>`__.\n\nauthors\n~~~~~~~\n\nThe authors of the package. This is a list of authors and should contain\nat least one author.\n\nAuthors must be in the form ``name <email>``.\n\nreadme\n~~~~~~\n\nThe readme file of the package. **Required**\n\nThe file can be either ``README.rst`` or ``README.md``. If it\'s a\nmarkdown file you have to install the\n`pandoc <https://github.com/jgm/pandoc>`__ utility so that it can be\nautomatically converted to a RestructuredText file.\n\nYou also need to have the\n`pypandoc <https://pypi.python.org/pypi/pypandoc/>`__ package installed.\nIf you install ``poet`` via ``pip`` you can use the ``markdown-readme``\nextra to do so.\n\n.. code:: bash\n\n    pip install pypoet[markdown-readme]\n\nhomepage\n~~~~~~~~\n\nAn URL to the website of the project. **Optional**\n\nrepository\n~~~~~~~~~~\n\nAn URL to the repository of the project. **Optional**\n\ndocumentation\n~~~~~~~~~~~~~\n\nAn URL to the documentation of the project. **Optional**\n\nkeywords\n~~~~~~~~\n\nA list of keywords (max: 5) that the package is related to. **Optional**\n\ninclude and exclude\n~~~~~~~~~~~~~~~~~~~\n\nA list of patterns that will be included in the final package.\n\nYou can explicitly specify to Poet that a set of globs should be ignored\nor included for the purposes of packaging. The globs specified in the\nexclude field identify a set of files that are not included when a\npackage is built.\n\nIf a VCS is being used for a package, the exclude field will be seeded\nwith the VCS’ ignore settings (``.gitignore`` for git for example).\n\n.. code:: toml\n\n    [package]\n    # ...\n    include = ["package/**/*.py", "package/**/.c"]\n\n.. code:: toml\n\n    exclude = ["package/excluded.py"]\n\n``dependencies`` and ``dev-dependencies``\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nPoetry is configured to look for dependencies on\n`PyPi <https://pypi.org>`__ by default. Only the name and a version\nstring are required in this case.\n\n.. code:: toml\n\n    [tool.poetry.dependencies]\n    requests = "^2.13.0"\n\nIf you want to use a private repository, you can add it to your\n``pyproject.toml`` file, like so:\n\n.. code:: toml\n\n    [[tool.poetry.source]]\n    name = \'private\'\n    url = \'http://example.com/simple\'\n\nBe aware that declaring the python version for which your package is\ncompatible is mandatory:\n\n.. code:: toml\n\n    [tool.poetry.dependencies]\n    python = "^3.6"\n\nCaret requirement\n^^^^^^^^^^^^^^^^^\n\n**Caret requirements** allow SemVer compatible updates to a specified\nversion. An update is allowed if the new version number does not modify\nthe left-most non-zero digit in the major, minor, patch grouping. In\nthis case, if we ran ``poetry update requests``, poetry would update us\nto version ``2.14.0`` if it was available, but would not update us to\n``3.0.0``. If instead we had specified the version string as\n``^0.1.13``, poetry would update to ``0.1.14`` but not ``0.2.0``.\n``0.0.x`` is not considered compatible with any other version.\n\nHere are some more examples of caret requirements and the versions that\nwould be allowed with them:\n\n.. code:: text\n\n    ^1.2.3 := >=1.2.3 <2.0.0\n    ^1.2 := >=1.2.0 <2.0.0\n    ^1 := >=1.0.0 <2.0.0\n    ^0.2.3 := >=0.2.3 <0.3.0\n    ^0.0.3 := >=0.0.3 <0.0.4\n    ^0.0 := >=0.0.0 <0.1.0\n    ^0 := >=0.0.0 <1.0.0\n\nTilde requirements\n^^^^^^^^^^^^^^^^^^\n\n**Tilde requirements** specify a minimal version with some ability to\nupdate. If you specify a major, minor, and patch version or only a major\nand minor version, only patch-level changes are allowed. If you only\nspecify a major version, then minor- and patch-level changes are\nallowed.\n\n``~1.2.3`` is an example of a tilde requirement.\n\n.. code:: text\n\n    ~1.2.3 := >=1.2.3 <1.3.0\n    ~1.2 := >=1.2.0 <1.3.0\n    ~1 := >=1.0.0 <2.0.0\n\nWildcard requirements\n^^^^^^^^^^^^^^^^^^^^^\n\n**Wildcard requirements** allow for any version where the wildcard is\npositioned.\n\n``*``, ``1.*`` and ``1.2.*`` are examples of wildcard requirements.\n\n.. code:: text\n\n    * := >=0.0.0\n    1.* := >=1.0.0 <2.0.0\n    1.2.* := >=1.2.0 <1.3.0\n\nInequality requirements\n^^^^^^^^^^^^^^^^^^^^^^^\n\n**Inequality requirements** allow manually specifying a version range or\nan exact version to depend on.\n\nHere are some examples of inequality requirements:\n\n.. code:: text\n\n    >= 1.2.0\n    > 1\n    < 2\n    != 1.2.3\n\nMultiple requirements\n^^^^^^^^^^^^^^^^^^^^^\n\nMultiple version requirements can also be separated with a comma, e.g.\n``>= 1.2, < 1.5``.\n\n``git`` dependencies\n^^^^^^^^^^^^^^^^^^^^\n\nTo depend on a library located in a ``git`` repository, the minimum\ninformation you need to specify is the location of the repository with\nthe git key:\n\n.. code:: toml\n\n    [dependencies]\n    requests = { git = "https://github.com/requests/requests.git" }\n\nSince we haven’t specified any other information, Poetry assumes that we\nintend to use the latest commit on the ``master`` branch to build our\nproject. You can combine the ``git`` key with the ``rev``, ``tag``, or\n``branch`` keys to specify something else. Here\'s an example of\nspecifying that you want to use the latest commit on a branch named\n``next``:\n\n.. code:: toml\n\n    [dependencies]\n    requests = { git = "https://github.com/kennethreitz/requests.git", branch = "next" }\n\nPython restricted dependencies\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nYou can also specify that a dependency should be installed only for\nspecific Python versions:\n\n.. code:: toml\n\n    [dependencies]\n    pathlib2 = { version = "^2.2", python = "~2.7" }\n\n.. code:: toml\n\n    [dependencies]\n    pathlib2 = { version = "^2.2", python = ["~2.7", "^3.2"] }\n\n``scripts``\n~~~~~~~~~~~\n\nThis section describe the scripts or executable that will be installed\nwhen installing the package\n\n.. code:: toml\n\n    [scripts]\n    poetry = \'poetry:console.run\'\n\nHere, we will have the ``poetry`` script installed which will execute\n``console.run`` in the ``poetry`` package.\n\n``extras``\n~~~~~~~~~~\n\nPoetry supports extras to allow expression of:\n\n-  optional dependencies, which enhance a package, but are not required;\n   and\n-  clusters of optional dependencies.\n\n.. code:: toml\n\n    [tool.poetry]\n    name = "awesome"\n\n    [tool.poetry.dependencies]\n    # These packages are mandatory and form the core of this package’s distribution.\n    mandatory = "^1.0"\n\n    # A list of all of the optional dependencies, some of which are included in the\n    # below `extras`. They can be opted into by apps.\n    psycopg2 = { version = "^2.7", optional = true }\n    mysqlclient = { version = "^1.3", optional = true }\n\n    [tool.poetry.extras]\n    mysql = ["mysqlclient"]\n    pgsql = ["psycopg2"]\n\nWhen installing packages, you can specify extras by using the\n``-E|--extras`` option:\n\n.. code:: bash\n\n    poet install --extras "mysql pgsql"\n    poet install -E mysql -E pgsql\n\n``plugins``\n~~~~~~~~~~~\n\nPoetry supports arbitrary plugins wich work similarly to `setuptools\nentry\npoints <http://setuptools.readthedocs.io/en/latest/setuptools.html>`__.\nTo match the example in the setuptools documentation, you would use the\nfollowing:\n\n.. code:: toml\n\n    [plugins] # Optional super table\n\n    [plugins."blogtool.parsers"]\n    ".rst" = "some_module::SomeClass"\n\nResources\n---------\n\n-  `Official Website <https://poetry.eustace.io>`__\n-  `Issue Tracker <https://github.com/sdispater/poetry/issues>`__\n',
    'author': 'Sébastien Eustace',
    'author_email': 'sebastien@eustace.io',
    'url': 'https://poetry.eustace.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>= 3.6.0.0, < 4.0.0.0',
}


setup(**setup_kwargs)
